package com.zcxk.syncdata.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSONObject;
import com.zcxk.syncdata.dao.*;
import com.zcxk.syncdata.dto.DeviceParams;
import com.zcxk.syncdata.entity.*;
import com.zcxk.syncdata.util.SnowflakeIdWorker;
import com.zoniot.platform.unit.management.model.UdipUnit;
import com.zcxk.syncdata.util.Util;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;

import com.zcxk.syncdata.service.DeviceService;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

import static com.google.common.collect.Lists.newArrayList;

@Slf4j
@Service
public class DeviceServiceImpl implements DeviceService {

    @Value("${files.path}")
    private String filesPath;

    @Resource
    private DeviceMapper deviceMapper;
    @Autowired
    UdipUnitRepository udipUnitRepository;
    @Autowired
    private SnowflakeIdWorker idWorker;
    @Resource
    private ConcentratorMapper concentratorMapper;
    @Resource
    private ChannelNumberMapper channelNumberMapper;
    @Resource
    private CommunityMapper communityMapper;
    @Resource
    private BuildingMapper buildingMapper;
    @Resource
    private WaterRelatedDeviceMapper waterRelatedDeviceMapper;
    @Resource
    private DeviceDimensionMapper deviceDimensionMapper;
    @Autowired
    private RemoteMeterDataListener remoteMeterDataListener;

    @Override
    public int insertSelective(Device record) {
        return deviceMapper.insertSelective(record);
    }

    @Override
    public int updateByPrimaryKeySelective(Device record) {
        return deviceMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public void syncData() {
        List<Device> list = deviceMapper.findList(1, 1, 1);
        if (list != null) {
            log.info("list size={}", list.size());
        }
    }

    private String getFilePath() {
        String name = Util.createUUIDId();
        String fullPath = filesPath + "/" + name + ".xlsm";
        return fullPath;
    }

    @Override
    public int importExcel(MultipartFile file) {
        log.info("begin importExcelBuilding");
        if (file != null) {
            String filePath = this.getFilePath();
            log.info("filePath =" + filePath);
            try {
                File uploadDir = new File(filePath);
                file.transferTo(uploadDir);
                InputStream inputStream = new FileInputStream(uploadDir);
                Workbook hssfWorkbook = null;
                //hssfWorkbook = new XSSFWorkbook(inputStream);//Excel 2007
                hssfWorkbook = new HSSFWorkbook(inputStream);//Excel 2007


                FormulaEvaluator evaluator = hssfWorkbook.getCreationHelper().createFormulaEvaluator();
                // 循环工作表Sheet
                for (int numSheet = 0; numSheet < hssfWorkbook.getNumberOfSheets(); numSheet++) {
                    Sheet hssfSheet = hssfWorkbook.getSheetAt(numSheet);
                    if (hssfSheet == null) {
                        continue;
                    }
                    List<Device> deviceList = newArrayList();
                    log.info("LastRowNum=" + hssfSheet.getLastRowNum());
                    // 循环行Row
                    for (int rowNum = 4; rowNum <= hssfSheet.getLastRowNum(); rowNum++) {
                    //for (int rowNum = 4; rowNum <= 10; rowNum++) {
                        Row row = hssfSheet.getRow(rowNum);
                        if (row != null) {
                            //小区 717
                            //建筑 6066
                            //客户 54


                            Cell deviceIdCell = row.getCell(0);
                            Cell nodeIdCell = row.getCell(2);

                            if (deviceIdCell == null) {
                                continue;
                            }

                            String deviceId = deviceIdCell.getStringCellValue();
                            String deviceNo = nodeIdCell.getStringCellValue();
                            //log.info("deviceId={},deviceNo={}",deviceId,deviceNo);
                            addMongo(deviceId, deviceNo);


                            Device device = new Device();
                            device.setId(idWorker.nextId());
                            device.setDeviceNo(deviceNo);
                            device.setDeviceType(20);
                            device.setSiteId(1);
                            device.setSysId(58);
                            device.setBuildingId(6066);
                            device.setLocDesc("");
                            device.setCustomerId(54);
                            device.setManufacturerId(8);
                            device.setDeviceStatus("5");
                            device.setStatus(true);
                            device.setUdipId(deviceId);
                            device.setCreateBy("system");
                            device.setUpdateBy("system");
                            device.setDateCreate(LocalDateTime.now());
                            device.setDateUpdate(LocalDateTime.now());
                            //log.info(JSON.toJSONString(device));
                            deviceList.add(device);


                        }
                    }
                    log.info("deviceList size" + deviceList.size());
                    if (deviceList.size() > 0) {
                        int total = deviceList.size();
                        int pageSize = 1000;
                        int totalPage = total / pageSize;
                        if (total % pageSize != 0) {
                            totalPage++;
                        }
                        for (int i = 1; i <= totalPage; i++) {
                            log.info("current page = " + i);
                            List<Device> records = newArrayList();
                            int start = (i - 1) * pageSize;
                            int j = 0;

                            for (int index = 0; (j < pageSize) && (index < total); j++) {
                                index = start + j;
                                records.add(deviceList.get(index));
                                index++;
                            }

                            int result = deviceMapper.batchInsert(records);
                            log.info("current page size = {} result={}" , records.size(),result);
                        }
                    }
                }

            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                //FileUtil.deleteFile(filePath);
            }
        } else {
            log.info("文件不能为空！");
        }
        return 0;
    }

    @Override
    public void addMongo(String id, String identifier) {
        UdipUnit unit = new UdipUnit();
        unit.setId(id);
        unit.setIdentifier(identifier);
        unit.setMode("NB-TJ2.0");
        unit.setType("water_meter");
        unit.setManufacturer("huaxu");
        unit.setAgentIdentifier("ct_nb_agent");
        unit.setStatus(1);
        unit.setComment("customerId:54");
        unit.setDateCreate(new Date());
        unit.setDateUpdate(new Date());
        udipUnitRepository.save(unit);
    }

    @Override
    public int batchInsert(List<Device> list) {
        return deviceMapper.batchInsert(list);
    }

    @Override
    public int importRemoteMeterExcel(MultipartFile file) {
        try {
            EasyExcel.read(file.getInputStream(), remoteMeterDataListener).sheet().headRowNumber(1).doRead();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return 0;
    }

    @Override
    public void batchImportDeviceHandler(DeviceParams deviceParams) {
        Integer siteId = 1;
        Integer customerId = 136;
        Integer province = 420000;
        Integer city = 420100;
        Integer region = 420107;
        Integer deviceTypeId = 19;
        String ip = "58.250.37.159";
        Integer port = 7000;


        Community community = communityMapper.findByNameV2(siteId,province,city,region,deviceParams.getCommunityName());
        if (community == null) {
            community = new Community();
            community.setSiteId(siteId);
            community.setCustomerId(customerId);
            community.setName(deviceParams.getCommunityName());
            community.setProvince(province);
            community.setCity(city);
            community.setRegion(region);
            community.setAddress(deviceParams.getCommunityName());
            community.setStatus(1);
            community.setCreateBy("system");
            community.setUpdateBy("system");
            community.setDateCreate(LocalDateTime.now());
            community.setDateUpdate(LocalDateTime.now());
            community.setCode(String.format("%03d",Integer.valueOf("1")));
            communityMapper.insertSelective(community);
        }

        String buildingName = "";
        if(!"NULL".equals(deviceParams.getUnitName())){
            buildingName = deviceParams.getBuildingName() + deviceParams.getUnitName();
        }
        Building building = buildingMapper.findByNameV2(siteId,community.getId(),buildingName);
        if (building == null) {
            building = new Building();
            building.setSiteId(siteId);
            building.setCommunity(community.getId());
            building.setName(buildingName);
            building.setProvince(province);
            building.setCity(city);
            building.setRegion(region);
            building.setAddress(community.getName()+buildingName);
            building.setStatus(1);
            building.setCreateBy("system");
            building.setUpdateBy("system");
            building.setCreateDate(LocalDateTime.now());
            building.setUpdateDate(LocalDateTime.now());
            buildingMapper.insertSelective(building);
        }

        Concentrator concentrator = concentratorMapper.findName(siteId,deviceParams.getConcentratorNo());
        if (concentrator == null) {
            concentrator = new Concentrator();
            concentrator.setSiteId(siteId);
            concentrator.setCustomerId(customerId);
            concentrator.setSerialNumber(deviceParams.getConcentratorNo());
            concentrator.setName(deviceParams.getConcentratorName());
            concentrator.setAddress(community.getName());
            concentrator.setDeviceType(deviceTypeId);
            concentrator.setBaudRate(1);
            concentrator.setIp(ip);
            concentrator.setPort(port);
            concentrator.setSimNo("1");
            concentrator.setBuildingId(building.getId());
            concentrator.setProvince(province);
            concentrator.setCity(city);
            concentrator.setRegion(region);
            concentrator.setDeviceStatus(0);
            concentrator.setStatus(1);
            concentrator.setCreateBy("system");
            concentrator.setUpdateBy("system");
            concentrator.setDateCreate(LocalDateTime.now());
            concentrator.setDateUpdate(LocalDateTime.now());
            concentratorMapper.insertSelective(concentrator);
        }
        ChannelNumber channelNumber = null;
        if(!"NULL".equals(deviceParams.getChannelNumberNo())){
            channelNumber = channelNumberMapper.findName(concentrator.getId(),deviceParams.getChannelNumberName());
            if (channelNumber == null) {
                channelNumber = new ChannelNumber();
                channelNumber.setSiteId(siteId);
                channelNumber.setConcentratorId(concentrator.getId());
                channelNumber.setChannelName(deviceParams.getChannelNumberName());
                channelNumber.setChannelNo(deviceParams.getChannelNumberNo());
                channelNumber.setStatus(1);
                channelNumber.setCreateBy("system");
                channelNumber.setUpdateBy("system");
                channelNumber.setDateCreate(LocalDateTime.now());
                channelNumber.setDateUpdate(LocalDateTime.now());
                channelNumberMapper.insertSelective(channelNumber);
            }
        }

        Device deviceTemp = deviceMapper.findByMeterNo(deviceParams.getMeterNo());
        if (deviceTemp == null) {
            Device device = new Device();
            device.setId(idWorker.nextId());
            device.setWaterMeterNo(deviceParams.getMeterNo());
            device.setWaterMeterFileNo(deviceParams.getFileNo());
            device.setDeviceType(deviceTypeId);
            device.setSiteId(siteId);
            device.setSysId(55);
            device.setBuildingId(building.getId());
            device.setLocDesc(deviceParams.getAddress());
            device.setCustomerId(customerId);
            device.setManufacturerId(8);
            device.setDeviceStatus("5");
            device.setStatus(true);
            device.setCreateBy("system");
            device.setUpdateBy("system");
            device.setDateCreate(LocalDateTime.now());
            device.setDateUpdate(LocalDateTime.now());
            deviceMapper.insertSelective(device);

            //2、同步设备维度关系
            DeviceDimension deviceDimension = new DeviceDimension();
            deviceDimension.setDeviceId(device.getId());
            deviceDimension.setProvince(String.valueOf(building.getProvince()));
            deviceDimension.setCity(String.valueOf(building.getCity()));
            deviceDimension.setRegion(String.valueOf(building.getRegion()));
            deviceDimension.setCommunity(String.valueOf(building.getCommunity()));
            deviceDimension.setBuilding(String.valueOf(building.getId()));
            deviceDimension.setBuilding(String.valueOf(building.getId()));
            deviceDimension.setDevice(device.getDeviceNo());
            deviceDimension.setCustomer(String.valueOf(device.getCustomerId()));
            deviceDimension.setStatus(1);
            deviceDimension.setCreateBy("system");
            deviceDimension.setUpdateBy("system");
            deviceDimension.setDateCreate(LocalDateTime.now());
            deviceDimension.setDateUpdate(LocalDateTime.now());
            deviceDimensionMapper.insertSelective(deviceDimension);

            //关系
            WaterRelatedDevice waterRelatedDevice = new WaterRelatedDevice();
            waterRelatedDevice.setDeviceId(device.getId());
            waterRelatedDevice.setConcentratorId(concentrator.getId());
            if (channelNumber != null) {
                waterRelatedDevice.setChannelNumberId(channelNumber.getId());
            }
            waterRelatedDevice.setPort(deviceParams.getPort());
            waterRelatedDevice.setIssueStatus(1);
            waterRelatedDevice.setStatus(1);
            waterRelatedDevice.setDateCreate(LocalDateTime.now());
            waterRelatedDevice.setDateUpdate(LocalDateTime.now());
            waterRelatedDevice.setCreateBy("system");
            waterRelatedDevice.setUpdateBy("system");
            waterRelatedDeviceMapper.insertSelective(waterRelatedDevice);
        }
    }
}


