package com.zcxk.syncdata.service.impl;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zcxk.syncdata.dao.*;
import com.zcxk.syncdata.dto.BuildingDto;
import com.zcxk.syncdata.dto.DeviceParams;
import com.zcxk.syncdata.entity.*;
import com.zcxk.syncdata.util.SnowflakeIdWorker;
import io.swagger.annotations.ApiModelProperty;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.nio.charset.Charset;
import java.time.LocalDateTime;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

@Slf4j
@Component
public class RemoteMeterDataListener extends AnalysisEventListener<Object> {
    @Resource
    private DeviceMapper deviceMapper;
    @Resource
    private ConcentratorMapper concentratorMapper;
    @Resource
    private ChannelNumberMapper channelNumberMapper;
    @Resource
    private CommunityMapper communityMapper;
    @Resource
    private BuildingMapper buildingMapper;
    @Resource
    private WaterRelatedDeviceMapper waterRelatedDeviceMapper;
    @Resource
    private DeviceDimensionMapper deviceDimensionMapper;
    @Autowired
    private SnowflakeIdWorker idWorker;
    @Resource
    private RabbitTemplate rabbitTemplate;
    @Value("${spring.rabbitmq.exchange}")
    private String exchange;
    @Value("${spring.rabbitmq.queue}")
    private String queue;

    AtomicInteger success = new AtomicInteger(0);
    AtomicInteger fail = new AtomicInteger(0);
    private static final Integer siteId = 1;
    private static final Integer customerId = 136;
    private static final Integer province = 420000;
    private static final Integer city = 420100;
    private static final Integer region = 420107;
    private static final Integer deviceTypeId = 19;


    @Override
    public void invoke(Object o, AnalysisContext analysisContext) {
        Map<Integer,String> map = (Map<Integer, String>) o;
        log.info("解析到一条数据:{}", JSON.toJSONString(map));

        String meterNo = map.get(0).trim();
        String fileNo = map.get(1).trim();
        String concentratorNo = map.get(2).trim();
        String concentratorName = map.get(3).trim();
        String channelNumberNo = map.get(4).trim();
        String channelNumberName = map.get(5).trim();
        String port = map.get(6).trim();
        String communityName = map.get(7).trim();
        String buildingName = map.get(8).trim();
        String unitName = map.get(9).trim();
        String address = map.get(10).trim();

        DeviceParams deviceParams = new DeviceParams();
        deviceParams.setBatchId(1);
        deviceParams.setMeterNo(meterNo);
        deviceParams.setFileNo(fileNo);
        deviceParams.setConcentratorNo(concentratorNo);
        deviceParams.setConcentratorName(concentratorName);
        deviceParams.setChannelNumberNo(channelNumberNo);
        deviceParams.setChannelNumberName(channelNumberName);
        deviceParams.setPort(port);
        deviceParams.setCommunityName(communityName);
        deviceParams.setBuildingName(buildingName);
        deviceParams.setUnitName(unitName);
        deviceParams.setAddress(address);

        String msg = JSON.toJSONString(deviceParams);
        log.info("transfer mq msg:{}",msg);
        rabbitTemplate.convertAndSend(exchange, queue,msg.getBytes(Charset.forName("UTF-8")));


        int result = success.incrementAndGet();
        log.info("finish={}",result);
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        log.info("所有数据解析完成！success={},fail={}",success.get(),fail.get());
    }
}
