package com.zcxk.syncdata.util.mongo;


import com.zcxk.syncdata.util.mongo.annotation.QueryField;
import com.zcxk.syncdata.util.mongo.annotation.UpdateField;
import com.zcxk.syncdata.util.mongo.customized.CustomizedUpdate;
import org.springframework.data.mongodb.core.query.Query;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * <p></p>
 *
 * @Author wilian.peng
 * @Date 2020/3/24 19:57
 * @Version 1.0
 */
public class MongoDaoSupport {
    /**
     * 根据vo构建查询条件Query
     * @param t
     * @return
     */
    public static Query buildBaseQuery(Object t) {
        Query query = new Query();

        List<Field> fields = getAllFields(t);
        for (Field field : fields) {
            field.setAccessible(true);
            try {
                Object value = field.get(t);
                if (value != null) {
                    QueryField queryField = field.getAnnotation(QueryField.class);
                    if (queryField != null) {
                        query.addCriteria(queryField.type().buildCriteria(queryField, field, value));
                    }
                }
            } catch (IllegalArgumentException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        }
        return query;
    }
    public static CustomizedUpdate buildBaseUpdate(Object t){
        CustomizedUpdate update = new CustomizedUpdate();
        List<Field> fields = getAllFields(t);
        for(Field field : fields){
            field.setAccessible(true);
            try {
                Object value = field.get(t);
                if (value != null) {
                    UpdateField updateField = field.getAnnotation(UpdateField.class);
                    if(updateField != null ){
                        update.addMultiFieldOperation(updateField.type().buildMultiFieldOperation(updateField,field,value));
                    }
                }
            }catch (IllegalArgumentException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        }
        return update ;
    }
    protected static List<Field> getAllFields(Object t) {
        List<Field> fieldList = new ArrayList<>() ;
        Class tempClass = t.getClass();
        while (tempClass != null) {//当父类为null的时候说明到达了最上层的父类(Object类).
            fieldList.addAll(Arrays.asList(tempClass .getDeclaredFields()));
            tempClass = tempClass.getSuperclass(); //得到父类,然后赋给自己
        }
        return fieldList;
    }
}
