package com.zoniot.platform.unit.management.model;

import com.zcxk.syncdata.util.mongo.annotation.QueryField;
import com.zcxk.syncdata.util.mongo.annotation.QueryType;
import com.zcxk.syncdata.util.mongo.annotation.UpdateField;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.data.mongodb.core.mapping.Document;

import java.io.Serializable;
import java.util.Date;

/**
 * <p> 集成元表,存储Mongodb,按照标识符进行分片</p>
 * sh.enableSharding("udip")
 * sh.shardCollection("udip.udip_unit_t",{"identifier":"hashed"});
 * @Author wilian.peng
 * @Date 2020/3/20 17:55
 * @Version 1.0
 */
@ApiModel(value="集成元")
@Data
@NoArgsConstructor
@Document(collection = "udip_unit_t")
public class UdipUnit implements Serializable {

    @ApiModelProperty(value="主键")
    @QueryField
    private String id;

    @ApiModelProperty(value="标识符")
    @QueryField(type = QueryType.EQUALS_IGNORE_CASE)
    @UpdateField
    private String identifier;

    @ApiModelProperty(value="型号")
    @QueryField
    @UpdateField
    private String mode;

    @ApiModelProperty(value="类型")
    @QueryField
    @UpdateField
    private String type;

    @ApiModelProperty(value="厂商")
    @QueryField
    @UpdateField
    private String manufacturer;

    @ApiModelProperty(value="关联Profile")
    @UpdateField
    private Integer profileId;

    @ApiModelProperty(value="关联Agent")
    @QueryField
    @UpdateField
    private String agentIdentifier;

    @ApiModelProperty(value="集成方唯一标识符")
    private String integrationIdentifier;

    @ApiModelProperty(value="当前状态")
    @QueryField
    @UpdateField
    private Integer unitStatus;

    @ApiModelProperty(value="最后更新时间")
    @UpdateField
    private Date lasttime;

    @ApiModelProperty(value="状态")
    @UpdateField
    private Integer status;

    @UpdateField
    @ApiModelProperty(value="备注")
    private String comment;

    @ApiModelProperty(value="创建人")
    private String createBy;

    @ApiModelProperty(value="创建时间")
    private Date dateCreate = new Date();

    @ApiModelProperty(value="更新时间")
    //@UpdateField(type= UpdateType.CURRENTTIMESTAMP)
    private Date dateUpdate  = new Date();

    @ApiModelProperty(value="更新人")
    private String updateBy;

    private static final long serialVersionUID = 1L;
}